import { ComponentType } from "react";
import { PluginDefinition } from "../PluginDefinition";
export interface ExpandedRowProps<T> {
    item: T;
}
export declare type ExpandableRowsPluginOptions<T> = {
    [K in keyof T]: {
        /**
         * Key of object to use to determine expanded status. Ideally this should
         * be a unique field for each item, such as an ID.
         */
        expansionKey: K;
        /**
         * Component rendered inside expanded rows
         */
        component: ComponentType<ExpandedRowProps<T>>;
        /**
         * Determines whether or not the expanded content will be rendered
         * above or below the data row. Default "below".
         */
        expansionDirection?: "below" | "above";
        /**
         * Values at the `expansionKey` for each expanded item. A strict equality
         * check is applied by default. Supply `predicate` to override this
         * behaviour.
         */
        expandedValues: T[K][];
        /**
         * Values at the `expansionKey` for items whose rows can't be expanded.
         */
        disabledValues?: T[K][];
        /**
         * Called when any selection component is clicked.
         * Use to update selected items
         *
         * @param newExpandedItemValues Array representing the rows that are
         * currently expanded
         * @param difference The intersection between the input `expandedValues`
         * and the target `newExpandedItemValues`
         */
        onRowToggle: (newExpandedItemValues: T[K][], difference: T[K], action: "expanding" | "collapsing") => void | Promise<void>;
        /**
         * Maximum number of rows which can be expanded at once.
         * Default: 1
         * For infinite, supply "infinite"
         */
        maxExpandedRows?: number | "infinite";
        /**
         * Injects a column containing a caret to indicate the row's expansion
         * status
         */
        displayColumnToggleCaret?: boolean;
        /** Overrides the default caret display */
        columnToggleCaretComponent?: ComponentType<{
            expanded: boolean;
        }>;
        /**
         * Overrides the position of the toggle caret column. Defaults to "end"
         */
        columnToggleCaretColumnPosition?: number | "start" | "end";
    };
}[keyof T];
declare module "../../plugins/PluginDefinition" {
    interface TablePluginOptions<T> {
        expandableRows?: ExpandableRowsPluginOptions<T>;
    }
    interface ColumnPluginOptions<T, K extends keyof T> {
        expandableRows?: {
            isCaretColumn: boolean;
        };
    }
}
export declare const ExpandableRowsPlugin: PluginDefinition<any>;
