"use strict";
var __spreadArray = (this && this.__spreadArray) || function (to, from, pack) {
    if (pack || arguments.length === 2) for (var i = 0, l = from.length, ar; i < l; i++) {
        if (ar || !(i in from)) {
            if (!ar) ar = Array.prototype.slice.call(from, 0, i);
            ar[i] = from[i];
        }
    }
    return to.concat(ar || Array.prototype.slice.call(from));
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ExpandableRowsPlugin = void 0;
var morse_react_1 = require("morse-react");
var react_1 = __importDefault(require("react"));
exports.ExpandableRowsPlugin = {
    handleRowClick: function (item, pluginOptions) {
        var _a, _b;
        var expandableRows = pluginOptions === null || pluginOptions === void 0 ? void 0 : pluginOptions.expandableRows;
        // No expandable row configuration
        if (!expandableRows) {
            return;
        }
        var value = item[expandableRows.expansionKey];
        var isDisabled = (_a = expandableRows.disabledValues) === null || _a === void 0 ? void 0 : _a.includes(value);
        if (isDisabled) {
            return;
        }
        var maxExpandedRows = (_b = expandableRows.maxExpandedRows) !== null && _b !== void 0 ? _b : 1;
        var isExpanded = expandableRows.expandedValues.some(function (expandedValue) { return expandedValue === value; });
        if (isExpanded) {
            // Close
            expandableRows.onRowToggle(expandableRows.expandedValues.filter(function (expandedValue) { return expandedValue !== value; }), value, "collapsing");
        }
        else if (maxExpandedRows === "infinite" ||
            expandableRows.expandedValues.length < maxExpandedRows) {
            // Open
            expandableRows.onRowToggle(__spreadArray(__spreadArray([], expandableRows.expandedValues, true), [value], false), value, "expanding");
        }
        else {
            // Close in order rows were opened until open count + 1 is the same as the max expanded rows
            var difference = maxExpandedRows - expandableRows.expandedValues.length;
            expandableRows.onRowToggle(__spreadArray(__spreadArray([], expandableRows.expandedValues.slice(difference + 1), true), [value], false), value, "collapsing");
        }
    },
    postRowRender: function (renderedRow, item, columns, pluginOptions) {
        var _a;
        var expandableRows = pluginOptions === null || pluginOptions === void 0 ? void 0 : pluginOptions.expandableRows;
        // No expandable row configuration
        if (!expandableRows) {
            return;
        }
        var isExpanded = expandableRows.expandedValues.includes(item[expandableRows.expansionKey]);
        if (isExpanded) {
            var direction = (_a = expandableRows.expansionDirection) !== null && _a !== void 0 ? _a : "below";
            var expandedRow = (react_1.default.createElement(morse_react_1.TableRow, { key: "expanded-row-".concat(item[expandableRows.expansionKey]) },
                react_1.default.createElement(morse_react_1.TableCell, { colSpan: columns.length },
                    react_1.default.createElement(expandableRows.component, { item: item }))));
            if (direction === "below") {
                return (react_1.default.createElement(react_1.default.Fragment, null,
                    renderedRow,
                    expandedRow));
            }
            else {
                return (react_1.default.createElement(react_1.default.Fragment, null,
                    expandedRow,
                    renderedRow));
            }
        }
        else {
            return renderedRow;
        }
    },
    cellRender: function (item, columnDefinition, tablePluginOptions) {
        var _a, _b, _c;
        if (!((_b = (_a = columnDefinition.pluginOptions) === null || _a === void 0 ? void 0 : _a.expandableRows) === null || _b === void 0 ? void 0 : _b.isCaretColumn)) {
            return false;
        }
        var expandableRows = tablePluginOptions === null || tablePluginOptions === void 0 ? void 0 : tablePluginOptions.expandableRows;
        // No expandable row configuration
        if (!expandableRows) {
            return;
        }
        var value = item[expandableRows.expansionKey];
        var isExpanded = expandableRows.expandedValues.some(function (expandedValue) { return expandedValue === value; });
        if ((_c = tablePluginOptions === null || tablePluginOptions === void 0 ? void 0 : tablePluginOptions.expandableRows) === null || _c === void 0 ? void 0 : _c.columnToggleCaretComponent) {
            return (react_1.default.createElement(tablePluginOptions.expandableRows.columnToggleCaretComponent, { expanded: isExpanded }));
        }
        else {
            return isExpanded ? "⮟" : "⮞";
        }
    },
    injectedColumns: function (pluginOptions) {
        var _a;
        return [
            {
                definition: {
                    // Column pluginOptions
                    pluginOptions: {
                        expandableRows: {
                            isCaretColumn: true,
                        },
                    },
                },
                position: (_a = pluginOptions === null || pluginOptions === void 0 ? void 0 : pluginOptions.expandableRows) === null || _a === void 0 ? void 0 : _a.columnToggleCaretColumnPosition,
            },
        ];
    },
};
//# sourceMappingURL=ExpandableRowsPlugin.js.map