import React from "react";
import { ComponentType } from "react";
import { PluginDefinition } from "../PluginDefinition";
interface BaseSelectionComponentProps {
    selected: boolean;
    onClick: (e: React.MouseEvent) => void;
}
export interface SelectionComponentProps<T, K extends keyof T> extends BaseSelectionComponentProps {
    item: T;
    value: K;
    disabled?: boolean;
}
export interface SelectAllComponentProps extends BaseSelectionComponentProps {
}
export declare type SelectableRowsPluginOptions<T> = {
    [K in keyof T]: {
        /**
         * Key of object to use to determine selection status. Ideally this should
         * be a unique field for each item, such as an ID.
         */
        selectionKey: K;
        /**
         * Values at the `selectionKey` for each selected item. A strict equality
         * check is applied by default. Supply `predicate` to override this
         * behaviour.
         */
        selectedValues: T[K][];
        /**
         * Values at the `selectionKey` for items whose selection components should
         * be disabled.
         */
        disabledValues?: T[K][];
        /**
         * Called when any selection component is clicked.
         * Use to update selected items
         */
        onSelection: (selectedItems: T[K][]) => void;
        /** Called when the select all component is clicked. */
        onSelectAll?: (allSelected: boolean) => void;
        /**
         * By default a strict equality check is applied to determine if a given
         * item is selected. Supply a predicate to override this behaviour.
         */
        predicate?: (value: T[K], itemUnderTest: T) => boolean;
        /**
         * By default a strict equality check is applied to determine if a given
         * item is disabled. Supply a predicate to override this behaviour.
         */
        disabledPredicate?: (value: T[K], itemUnderTest: T) => boolean;
        canSelectAll?: boolean;
        /**
         * Overrides component rendered inside each row's data cell. This component
         * is also used in the heading row is `canSelectAll` is true, and no custom
         * `selectAllComponent` is supplied.
         */
        component?: ComponentType<SelectionComponentProps<T, K>>;
        /**
         * Class name applied to the default checkbox component if `component`
         * isn't supplied.
         */
        className?: string;
        /**
         * Overrides component rendered inside the heading row if `canSelectAll`
         * is true
         */
        selectAllComponent?: ComponentType<SelectAllComponentProps>;
    };
}[keyof T];
declare module "../../plugins/PluginDefinition" {
    interface TablePluginOptions<T> {
    }
    interface ColumnPluginOptions<T, K extends keyof T> {
        selectableRows?: SelectableRowsPluginOptions<T>;
    }
}
export declare const SelectableRowsPlugin: PluginDefinition<any>;
export {};
